RwLockReadGuard

Struct RwLockReadGuard 

pub struct RwLockReadGuard<'a, T>
where T: ?Sized,
{ pub(super) s: &'a Semaphore, pub(super) data: *const T, pub(super) marker: PhantomData<&'a T>, }
RAII structure used to release the shared read access of a lock when dropped.

This structure is created by the read method on RwLock.

Fields

s: &'a Semaphoredata: *const Tmarker: PhantomData<&'a T>

Implementations



impl<'a, T> RwLockReadGuard<'a, T>
where T: ?Sized,

pub fn map<F, U>(this: RwLockReadGuard<'a, T>, f: F) -> RwLockReadGuard<'a, U>
where F: FnOnce(&T) -> &U, U: ?Sized,

Makes a new RwLockReadGuard for a component of the locked data.

This operation cannot fail as the RwLockReadGuard passed in already locked the data.

This is an associated function that needs to be used as RwLockReadGuard::map(...). A method would interfere with methods of the same name on the contents of the locked data.

This is an asynchronous version of RwLockReadGuard::map from the parking_lot crate.

Examples
use tokio::sync::{RwLock, RwLockReadGuard};

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
struct Foo(u32);

let lock = RwLock::new(Foo(1));

let guard = lock.read().await;
let guard = RwLockReadGuard::map(guard, |f| &f.0);

assert_eq!(1, *guard);

pub fn try_map<F, U>( this: RwLockReadGuard<'a, T>, f: F, ) -> Result<RwLockReadGuard<'a, U>, RwLockReadGuard<'a, T>>
where F: FnOnce(&T) -> Option<&U>, U: ?Sized,

Attempts to make a new RwLockReadGuard for a component of the locked data. The original guard is returned if the closure returns None.

This operation cannot fail as the RwLockReadGuard passed in already locked the data.

This is an associated function that needs to be used as RwLockReadGuard::try_map(..). A method would interfere with methods of the same name on the contents of the locked data.

This is an asynchronous version of RwLockReadGuard::try_map from the parking_lot crate.

Examples
use tokio::sync::{RwLock, RwLockReadGuard};

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
struct Foo(u32);

let lock = RwLock::new(Foo(1));

let guard = lock.read().await;
let guard = RwLockReadGuard::try_map(guard, |f| Some(&f.0)).expect("should not fail");

assert_eq!(1, *guard);