Struct MutexGuard
pub struct MutexGuard<'a, T>where
T: ?Sized,{
lock: &'a Mutex<T>,
}Expand description
A handle to a held Mutex. The guard can be held across any .await point
as it is Send.
As long as you have this guard, you have exclusive access to the underlying
T. The guard internally borrows the Mutex, so the mutex will not be
dropped while a guard exists.
The lock is automatically released whenever the guard is dropped, at which
point lock will succeed yet again.
Fields
lock: &'a Mutex<T>Implementations
impl<'a, T> MutexGuard<'a, T>where
T: ?Sized,
impl<'a, T> MutexGuard<'a, T>where
T: ?Sized,
pub fn map<U, F>(this: MutexGuard<'a, T>, f: F) -> MappedMutexGuard<'a, U>
pub fn map<U, F>(this: MutexGuard<'a, T>, f: F) -> MappedMutexGuard<'a, U>
Makes a new MappedMutexGuard for a component of the locked data.
This operation cannot fail as the MutexGuard passed in already locked the mutex.
This is an associated function that needs to be used as MutexGuard::map(...). A method
would interfere with methods of the same name on the contents of the locked data.
Examples
use tokio::sync::{Mutex, MutexGuard};
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
struct Foo(u32);
let foo = Mutex::new(Foo(1));
{
let mut mapped = MutexGuard::map(foo.lock().await, |f| &mut f.0);
*mapped = 2;
}
assert_eq!(Foo(2), *foo.lock().await);pub fn try_map<U, F>(
this: MutexGuard<'a, T>,
f: F,
) -> Result<MappedMutexGuard<'a, U>, MutexGuard<'a, T>>
pub fn try_map<U, F>( this: MutexGuard<'a, T>, f: F, ) -> Result<MappedMutexGuard<'a, U>, MutexGuard<'a, T>>
Attempts to make a new MappedMutexGuard for a component of the locked data. The
original guard is returned if the closure returns None.
This operation cannot fail as the MutexGuard passed in already locked the mutex.
This is an associated function that needs to be used as MutexGuard::try_map(...). A
method would interfere with methods of the same name on the contents of the locked data.
Examples
use tokio::sync::{Mutex, MutexGuard};
#[derive(Debug, Clone, Copy, PartialEq, Eq)]
struct Foo(u32);
let foo = Mutex::new(Foo(1));
{
let mut mapped = MutexGuard::try_map(foo.lock().await, |f| Some(&mut f.0))
.expect("should not fail");
*mapped = 2;
}
assert_eq!(Foo(2), *foo.lock().await);pub fn mutex(this: &MutexGuard<'a, T>) -> &'a Mutex<T>
pub fn mutex(this: &MutexGuard<'a, T>) -> &'a Mutex<T>
Returns a reference to the original Mutex.
use tokio::sync::{Mutex, MutexGuard};
async fn unlock_and_relock<'l>(guard: MutexGuard<'l, u32>) -> MutexGuard<'l, u32> {
println!("1. contains: {:?}", *guard);
let mutex = MutexGuard::mutex(&guard);
drop(guard);
let guard = mutex.lock().await;
println!("2. contains: {:?}", *guard);
guard
}Trait Implementations
impl<T> Debug for MutexGuard<'_, T>
impl<T> Debug for MutexGuard<'_, T>
impl<T> DerefMut for MutexGuard<'_, T>where
T: ?Sized,
impl<T> DerefMut for MutexGuard<'_, T>where
T: ?Sized,
fn deref_mut(&mut self) -> &mut <MutexGuard<'_, T> as Deref>::Target
fn deref_mut(&mut self) -> &mut <MutexGuard<'_, T> as Deref>::Target
impl<T> Display for MutexGuard<'_, T>
impl<T> Display for MutexGuard<'_, T>
impl<T> Drop for MutexGuard<'_, T>where
T: ?Sized,
impl<T> Drop for MutexGuard<'_, T>where
T: ?Sized,
impl<T> Deref for MutexGuard<'_, T>where
T: ?Sized,
impl<T> Deref for MutexGuard<'_, T>where
T: ?Sized,
impl<T> Sync for MutexGuard<'_, T>